{% extends "furo/page.html" %}

{% block extrahead %}
<!-- Google tag (gtag.js) -->
<script async src="https://www.googletagmanager.com/gtag/js?id=G-KVVHD37BKD"></script>
<script>
  window.dataLayer = window.dataLayer || [];
  function gtag(){dataLayer.push(arguments);}
  gtag('js', new Date());
  gtag('config', 'G-KVVHD37BKD');
</script>
{% endblock %}

{% block footer %}
        <div class="related-pages">
          {% if next -%}
            <a class="next-page" href="{{ next.link }}">
              <div class="page-info">
                <div class="context">
                  <span>{{ _("Next") }}</span>
                </div>
                <div class="title">{{ next.title }}</div>
              </div>
              <svg class="furo-related-icon"><use href="#svg-arrow-right"></use></svg>
            </a>
          {%- endif %}
          {% if prev -%}
            <a class="prev-page" href="{{ prev.link }}">
              <svg class="furo-related-icon"><use href="#svg-arrow-right"></use></svg>
              <div class="page-info">
                <div class="context">
                  <span>{{ _("Previous") }}</span>
                </div>
                {% if prev.link == pathto(master_doc) %}
                <div class="title">{{ _("Home") }}</div>
                {% else %}
                <div class="title">{{ prev.title }}</div>
                {% endif %}
              </div>
            </a>
          {%- endif %}
        </div>
        <div class="bottom-of-page">
          <div class="left-details">
            {%- if show_copyright %}
            <div class="copyright">
              {%- if hasdoc('copyright') %}
                {% trans path=pathto('copyright'), copyright=copyright|e -%}
                  <a href="{{ path }}">Copyright</a> &#169; {{ copyright }} | <a href="/qluster">Qluster</a> | <a href="https://zepworks.com">Zepworks Blog</a>
                {%- endtrans %}
              {%- else %}
                {% trans copyright=copyright|e -%}
                  Copyright &#169; {{ copyright }} | <a href="/qluster">Qluster</a> | <a href="https://zepworks.com">Zepworks Blog</a>
                {%- endtrans %}
              {%- endif %}
            </div>
            {%- endif %}
          </div>
          <div class="right-details">
            {% if theme_footer_icons -%}
            <div class="icons">
              {% for icon_dict in theme_footer_icons -%}
              <a class="muted-link {{ icon_dict.class }}" href="{{ icon_dict.url }}" aria-label="{{ icon_dict.name }}">
                {{- icon_dict.html -}}
              </a>
              {% endfor %}
            </div>
            {%- endif %}
          </div>
        </div>
{% endblock %}
