#!/usr/bin/env python3
#
# DeepDiff documentation build configuration file, created by
# sphinx-quickstart on Mon Jul 20 06:06:44 2015.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import sys
import os
import datetime
from dotenv import load_dotenv

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
# import pdb; pdb.set_trace()
sys.path.insert(0, os.path.abspath('..'))

# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx_sitemap',
    'sphinxemoji.sphinxemoji',
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The encoding of source files.
#source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = 'index'

# General information about the project.

year = datetime.datetime.now().year
project = 'DeepDiff'
copyright = '2015-{}, Sep Dehpour'.format(year)
author = 'Sep Dehpour'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = '9.0.0'
# The full version, including alpha/beta/rc tags.
release = '9.0.0'

load_dotenv(override=True)
DOC_VERSION = os.environ.get('DOC_VERSION', version)


# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'en'

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
#today = ''
# Else, today_fmt is used as the format for a strftime call.
#today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ['_build']

# The reST default role (used for this markup: `text`) to use for all
# documents.
#default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
#add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
#add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
#show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
# Let Furo use its default pygments styles (light + dark mode aware)
pygments_style = 'xcode'
pygments_dark_style = 'github-dark'

# A list of ignored prefixes for module index sorting.
#modindex_common_prefix = []

# If true, keep warnings as "system message" paragraphs in the built documents.
#keep_warnings = False

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'furo'

html_theme_options = {
    'footer_icons': [
        {
            'name': 'GitHub',
            'url': 'https://github.com/seperman/deepdiff',
            'html': '<svg stroke="currentColor" fill="currentColor" stroke-width="0" '
                    'viewBox="0 0 16 16"><path fill-rule="evenodd" d="M8 0C3.58 0 0 3.58 0 8c0 '
                    '3.54 2.29 6.53 5.47 7.59.4.07.55-.17.55-.38 0-.19-.01-.82-.01-1.49-2.01.37-'
                    '2.53-.49-2.69-.94-.09-.23-.48-.94-.82-1.13-.28-.15-.68-.52-.01-.53.63-.01 '
                    '1.08.58 1.23.82.72 1.21 1.87.87 2.33.66.07-.52.28-.87.51-1.07-1.78-.2-'
                    '3.64-.89-3.64-3.95 0-.87.31-1.59.82-2.15-.08-.2-.36-1.02.08-2.12 0 0 '
                    '.67-.21 2.2.82.64-.18 1.32-.27 2-.27.68 0 1.36.09 2 .27 1.53-1.04 2.2-'
                    '.82 2.2-.82.44 1.1.16 1.92.08 2.12.51.56.82 1.27.82 2.15 0 3.07-1.87 '
                    '3.75-3.65 3.95.29.25.54.73.54 1.48 0 1.07-.01 1.93-.01 2.2 0 .21.15.46'
                    '.55.38A8.013 8.013 0 0016 8c0-4.42-3.58-8-8-8z"></path></svg>',
            'class': '',
        },
    ],
    'source_repository': 'https://github.com/seperman/deepdiff',
    'source_branch': 'master',
    'source_directory': 'docs/',
    'light_css_variables': {
        # Fonts
        'font-stack': "'Open Sans', sans-serif",
        'font-stack--headings': "'Open Sans', sans-serif",
        'font-size--normal': '17px',
        # Body colors (from Alabaster)
        'color-foreground-primary': '#3E4349',
        'color-foreground-secondary': '#555',
        'color-foreground-muted': '#888',
        'color-background-primary': '#fff',
        'color-background-secondary': '#f8f8f8',
        'color-background-border': '#EEE',
        # Links
        'color-link': '#004B6B',
        'color-link--hover': '#6D4100',
        # Sidebar
        'color-sidebar-background': '#fff',
        'color-sidebar-text': '#555',
        'color-sidebar-link': '#444',
        'color-sidebar-link-text': '#444',
        'color-sidebar-link-text--top-level': '#444',
        'color-sidebar-item-background--hover': '#EEE',
        'color-sidebar-item-expander-color': '#444',
        'color-sidebar-caption-text': '#444',
        'color-sidebar-item-background--current': 'transparent',
        'color-sidebar-search-border': '#CCC',
        # Code blocks
        'color-code-background': '#ecf0f3',
        'color-code-foreground': '#222',
        'color-inline-code-background': '#ecf0f3',
        # Admonitions
        'color-admonition-background': '#EEE',
        'color-admonition-title': '#195190',
        'color-admonition-title-background': 'rgba(25, 81, 144, 0.1)',
        # Table of contents (right sidebar)
        'color-toc-item-text': '#444',
        'color-toc-item-text--hover': '#195190',
        'color-toc-item-text--active': '#195190',
        # Headerlink
        'color-header-text': '#3E4349',
    },
    'dark_css_variables': {
        'font-stack': "'Open Sans', sans-serif",
        'font-stack--headings': "'Open Sans', sans-serif",
        'font-size--normal': '17px',
        # Links
        'color-link': '#7EB8DA',
        'color-link--hover': '#D4A76A',
        # Sidebar
        'color-sidebar-background': '#1a1c1e',
        'color-sidebar-background-border': '#2d2d2d',
        'color-sidebar-link-text': '#ccc',
        'color-sidebar-link-text--top-level': '#ccc',
        'color-sidebar-caption-text': '#ccc',
        'color-sidebar-item-background--hover': '#2d2d2d',
        # Code blocks
        'color-code-background': '#2b2b2b',
        'color-inline-code-background': '#2b2b2b',
        # Admonitions
        'color-admonition-title': '#7EB8DA',
        'color-admonition-title-background': 'rgba(126, 184, 218, 0.15)',
        # TOC
        'color-toc-item-text--hover': '#7EB8DA',
        'color-toc-item-text--active': '#7EB8DA',
    },
}

# Add any paths that contain custom themes here, relative to this directory.
#html_theme_path = []

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
#html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
#html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
html_logo = '_static/qluster_grey_on_white_bg_optimized.svg'

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
html_favicon = "./_static/favicon.ico"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']
html_css_files = ['custom.css']

# Add any extra paths that contain custom files (such as robots.txt or
# .htaccess) here, relative to this directory. These files are copied
# directly to the root of the documentation.
#html_extra_path = []

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
#html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
#html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
#html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
#html_additional_pages = {}

# If false, no module index is generated.
#html_domain_indices = True

# If false, no index is generated.
#html_use_index = True

# If true, the index is split into individual pages for each letter.
#html_split_index = False

# If true, links to the reST sources are added to the pages.
#html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
#html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
#html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
#html_file_suffix = None

# Language to be used for generating the HTML full-text search index.
# Sphinx supports the following languages:
#   'da', 'de', 'en', 'es', 'fi', 'fr', 'h', 'it', 'ja'
#   'nl', 'no', 'pt', 'ro', 'r', 'sv', 'tr'
html_search_language = 'en'

# A dictionary with options for the search language support, empty by default.
# Now only 'ja' uses this config value
#html_search_options = {'type': 'default'}

# The name of a javascript file (relative to the configuration directory) that
# implements a search results scorer. If empty, the default will be used.
#html_search_scorer = 'scorer.js'

# Output file base name for HTML help builder.
htmlhelp_basename = 'DeepDiffdoc'

# Sitemap settings:

html_baseurl = f'https://zepworks.com/deepdiff/{DOC_VERSION}'
sitemap_url_scheme = "{link}"

# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
# The paper size ('letterpaper' or 'a4paper').
#'papersize': 'letterpaper',

# The font size ('10pt', '11pt' or '12pt').
#'pointsize': '10pt',

# Additional stuff for the LaTeX preamble.
#'preamble': '',

# Latex figure (float) alignment
#'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
  (master_doc, 'DeepDiff.tex', 'DeepDiff Documentation',
   'Sep Dehpour', 'manual'),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
#latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
#latex_use_parts = False

# If true, show page references after internal links.
#latex_show_pagerefs = False

# If true, show URL addresses after external links.
#latex_show_urls = False

# Documents to append as an appendix to all manuals.
#latex_appendices = []

# If false, no module index is generated.
#latex_domain_indices = True


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'deepdiff', 'DeepDiff Documentation',
     [author], 1)
]

# If true, show URL addresses after external links.
#man_show_urls = False


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
  (master_doc, 'DeepDiff', 'DeepDiff Documentation',
   author, 'DeepDiff', 'Deep difference of Python objects.',
   'Miscellaneous'),
]

# Documents to append as an appendix to all manuals.
#texinfo_appendices = []

# If false, no module index is generated.
#texinfo_domain_indices = True

# How to display URL addresses: 'footnote', 'no', or 'inline'.
#texinfo_show_urls = 'footnote'

# If true, do not generate a @detailmenu in the "Top" node's menu.
#texinfo_no_detailmenu = False
