@import url('https://fonts.googleapis.com/css2?family=Open+Sans:ital,wght@0,300..800;1,300..800&display=swap');

h1, h2, h3, h4, h5, h6 {
  font-weight: normal;
}

/* Sidebar brand: logo links externally, text links to docs root */
.sidebar-brand {
  display: flex;
  flex-direction: column;
  align-items: center;
  padding: var(--sidebar-item-spacing-vertical) var(--sidebar-item-spacing-horizontal);
}
.sidebar-brand-text {
  overflow-wrap: anywhere;
  text-decoration: none;
  color: var(--color-sidebar-brand-text, var(--color-foreground-primary));
  font-size: 1.5rem;
}
.sidebar-brand-text:visited {
  color: #2A6496;
}
.sidebar-brand-text:hover {
  color: var(--color-sidebar-brand-text, var(--color-foreground-primary));
}

/* Sidebar active/current item */
.sidebar-tree .current-page > .reference {
  color: #195190;
}

/* Right-side TOC active link */
.toc-tree a.active {
  color: #195190;
}

/* Dark mode overrides */
@media (prefers-color-scheme: dark) {
  body[data-theme="auto"] .sidebar-tree .current-page > .reference,
  body[data-theme="dark"] .sidebar-tree .current-page > .reference {
    color: #7EB8DA;
  }
  body[data-theme="auto"] .toc-tree a.active,
  body[data-theme="dark"] .toc-tree a.active {
    color: #7EB8DA;
  }
  body[data-theme="auto"] .sidebar-brand img {
    filter: brightness(0) invert(1);
  }
}

body[data-theme="dark"] .sidebar-tree .current-page > .reference {
  color: #7EB8DA;
}
body[data-theme="dark"] .toc-tree a.active {
  color: #7EB8DA;
}
body[data-theme="dark"] .sidebar-brand img {
  filter: brightness(0) invert(1);
}
